/* morla - Copyright (C) 2005-2006 bakunin - Andrea Marchesini 
 *                                     <bakunin@somasuite.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "editor.h"

/* Questa funzione viene eseguita quando si clicca su label o comment: */
gboolean
info_box_popup_show (GtkWidget * w, GdkEventButton * event, GList * list)
{
  GtkWidget *window;
  GtkWidget *frame;
  GtkWidget *vbox;
  GtkWidget *l;
  gint x, y;
  gchar *s;

  /* Se c'era una finestra prima... la rimuovo: */
  if ((window = g_object_get_data (G_OBJECT (w), "window")))
    gtk_widget_destroy (window);

  /* Creo la finestra con i suggerimenti: */
  window = gtk_window_new (GTK_WINDOW_POPUP);
  gtk_window_set_title (GTK_WINDOW (window), PACKAGE);
  gtk_window_set_type_hint (GTK_WINDOW (window),
			    GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
  gtk_window_set_decorated (GTK_WINDOW (window), FALSE);

  frame = gtk_frame_new (NULL);
  gtk_container_add (GTK_CONTAINER (window), frame);

  vbox = gtk_vbox_new (0, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  while (list)
    {
      struct info_box_t *d = list->data;

      if (d->lang)
	s = g_strdup_printf ("(%s) %s", d->lang, d->value);
      else
	s = g_strdup_printf ("%s", d->value);

      l = gtk_label_new (s);
      gtk_label_set_line_wrap (GTK_LABEL (l), TRUE);
      g_free (s);

      gtk_box_pack_start (GTK_BOX (vbox), l, FALSE, FALSE, 5);

      list = list->next;
    }

  /* La realizzo ocn le giuste dimensioni: */
  gtk_widget_show_all (frame);
  gtk_window_set_default_size (GTK_WINDOW (window), w->allocation.width, -1);
  gtk_widget_realize (window);

  /* Calcolo il centro del padre: */
  gdk_window_get_origin (w->window, &x, &y);
  x += (w->allocation.width / 2);
  y += (w->allocation.height / 2);

  /* Muovo la finestra nell'identico centro: */
  gtk_window_move (GTK_WINDOW (window), x - (window->allocation.width / 2),
		   y - (window->allocation.height / 2));

  gtk_widget_show_all (window);

  /* Salvo la finestra dentro al label: */
  g_object_set_data (G_OBJECT (w), "window", window);

  return TRUE;
}

/* Quando si molla il pulsante distruggo la finestra: */
gboolean
info_box_popup_hide (GtkWidget * w, GdkEventButton * event, gpointer dummy)
{
  GtkWidget *window = g_object_get_data (G_OBJECT (w), "window");

  if (window)
    gtk_widget_destroy (window);

  g_object_steal_data (G_OBJECT (w), "window");

  return TRUE;
}

/* Quando si entra nel box di suggerimento: */
gboolean
info_box_enter (GtkWidget * box, GdkEventCrossing * event, gpointer dummy)
{
  static GdkCursor *cursor = NULL;

  if (!cursor)
    cursor = gdk_cursor_new (GDK_HAND2);

  gdk_window_set_cursor (box->window, cursor);
  return TRUE;
}

/* Quando si esce dal box di suggerimento: */
gboolean
info_box_leave (GtkWidget * box, GdkEventCrossing * event, gpointer dummy)
{
  gdk_window_set_cursor (box->window, NULL);
  return TRUE;
}

void
info_box_free (struct info_box_t *i)
{
  if (!i)
    return;

  if (i->lang)
    g_free (i->lang);

  if (i->value)
    g_free (i->value);

  g_free (i);
}

/* EOF */
