/* morla - Copyright (C) 2005-2006 bakunin - Andrea Marchesini 
 *                                     <bakunin@somasuite.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "editor.h"

static gchar *rdf_trim (gchar * tmp);
static void rdf_parse_this (struct rdf_t *data, gchar * uri);

/* Questa funzione serve per ordinare liste di rdf con g_list_sort */
static gint
rdf_compare (struct rdf_t *a, struct rdf_t *b)
{
  return strcmp (a->subject, b->subject);
}

/* Questa funzione server per memorizzare dati di un nodo */
static void
rdf_node (gchar ** where, gint * type, gchar ** lang, librdf_node * node,
	  GList ** id)
{
  librdf_uri *uri;

  /* Che tipologia e'? */
  switch (librdf_node_get_type (node))
    {
    case LIBRDF_NODE_TYPE_RESOURCE:
      uri = librdf_node_get_uri (node);
      *where = (gchar *) librdf_uri_to_string (uri);
      *type = RDF_OBJECT_RESOURCE;
      break;

    case LIBRDF_NODE_TYPE_LITERAL:
      *where = (gchar *) librdf_node_get_literal_value (node);
      *lang = (gchar *) librdf_node_get_literal_value_language (node);
      *type = RDF_OBJECT_LITERAL;
      break;

    case LIBRDF_NODE_TYPE_BLANK:
      {
	gint ok = 0;
	GList *t;

	*where = (gchar *) librdf_node_get_blank_identifier (node);

	t = *id;
	ok = 0;

	while (t)
	  {
	    if (!strcmp ((gchar *) t->data, *where))
	      {
		ok = 1;
		break;
	      }
	    t = t->next;
	  }

	if (!ok)
	  *id = g_list_append (*id, *where);

	*type = RDF_OBJECT_BLANK;
      }

      break;

    default:
      break;
    }
}

/* Parso un documento RDF e ritorna una lista di triplette: */
GList *
rdf_parse (gchar * rdf, gboolean this_document)
{
  static librdf_world *world = NULL;
  librdf_storage *storage;
  librdf_parser *parser;
  librdf_model *model;
  librdf_stream *stream;
  librdf_uri *uri;
  GList *list;
  GList *blank = NULL;

  if (!world)
    {
      if (!(world = librdf_new_world ()))
	return NULL;

      librdf_world_open (world);
    }

  uri = librdf_new_uri (world, (const unsigned char *) rdf);
  if (!uri)
    return NULL;

  storage = librdf_new_storage (world, "memory", "test", NULL);
  if (!storage)
    {
      librdf_free_uri (uri);
      return NULL;
    }

  model = librdf_new_model (world, storage, NULL);
  if (!model)
    {
      librdf_free_storage (storage);
      librdf_free_uri (uri);
      return NULL;
    }

  parser = librdf_new_parser (world, NULL, NULL, NULL);
  if (!parser)
    {
      librdf_free_parser (parser);
      librdf_free_model (model);
      librdf_free_storage (storage);
      librdf_free_uri (uri);
      return NULL;
    }

  /* PARSE the URI as RDF/XML */
  if (librdf_parser_parse_into_model (parser, uri, uri, model))
    {
      librdf_free_parser (parser);
      librdf_free_model (model);
      librdf_free_storage (storage);
      librdf_free_uri (uri);
      return NULL;
    }

  stream = librdf_model_as_stream (model);
  if (!stream)
    {
      librdf_free_parser (parser);
      librdf_free_model (model);
      librdf_free_storage (storage);
      librdf_free_uri (uri);
      return NULL;
    }

  list = NULL;

  while (!librdf_stream_end (stream))
    {
      librdf_node *node;
      gint fake;
      librdf_statement *statement;
      struct rdf_t *new;

      if (!(statement = librdf_stream_get_object (stream)))
	break;

      new = (struct rdf_t *) g_malloc0 (sizeof (struct rdf_t));

      node = librdf_statement_get_subject (statement);
      rdf_node (&new->subject, &fake, &new->lang, node, &blank);
      if (!new->subject)
	{
	  rdf_free (new);
	  continue;
	}

      node = librdf_statement_get_predicate (statement);
      rdf_node (&new->predicate, &fake, &new->lang, node, &blank);
      if (!new->predicate)
	{
	  rdf_free (new);
	  continue;
	}

      node = librdf_statement_get_object (statement);
      rdf_node (&new->object, &fake, &new->lang, node, &blank);
      if (!new->object)
	{
	  rdf_free (new);
	  continue;
	}

      new->object_type = fake;

      new->subject = rdf_trim (new->subject);
      new->predicate = rdf_trim (new->predicate);
      new->object = rdf_trim (new->object);

      if (new->lang)
	new->lang = rdf_trim (new->lang);

      list = g_list_append (list, new);

      librdf_stream_next (stream);
    }

  /* Qui sostituisco i blank con qualcosa di piu' bellino a vedersi: */
  if (blank)
    {
      struct rdf_t *rdf_t;
      GList *tmp, *t;
      gint id = 0;

      tmp = blank;
      while (tmp)
	{

	  t = list;
	  while (t)
	    {
	      rdf_t = (struct rdf_t *) t->data;
	      if (!strcmp (rdf_t->subject, (gchar *) tmp->data))
		{
		  g_free (rdf_t->subject);
		  rdf_t->subject = g_strdup_printf (BLANK_NODE "%d", id);
		}

	      if (rdf_t->object_type == RDF_OBJECT_BLANK
		  && !strcmp (rdf_t->object, (gchar *) tmp->data))
		{
		  g_free (rdf_t->object);
		  rdf_t->object = g_strdup_printf (BLANK_NODE "%d", id);
		}

	      t = t->next;
	    }

	  id++;
	  tmp = tmp->next;
	}

      g_list_free (blank);
    }

  if (this_document == TRUE)
    g_list_foreach (list, (GFunc) rdf_parse_this, rdf);

  /* Libero memoria: */
  librdf_free_stream (stream);
  librdf_free_parser (parser);
  librdf_free_model (model);
  librdf_free_storage (storage);
  librdf_free_uri (uri);

  return g_list_sort (list, (GCompareFunc) rdf_compare);
}

/* Questa funzione sostituisce tutti i soggetti uguali all'URI del documento
 * con this:document: */
static void
rdf_parse_this (struct rdf_t *data, gchar * uri)
{
  if (!strcmp (data->subject, uri))
    {
      g_free (data->subject);
      data->subject = g_strdup (THIS_DOCUMENT);
    }
}

/* Questo documento RDF e' salvato? */
gint
rdf_is_no_saved (struct editor_data_t *data)
{
  return data->changed;
}

/* Funzione di trim: */
static gchar *
rdf_trim (gchar * tmp)
{
  gchar *ret;
  gint i, j, q;
  gint size;

  i = 0;
  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    tmp++;

  i = strlen (tmp);
  i--;

  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    i--;

  tmp[i + 1] = 0;

  size = strlen (tmp);
  ret = g_malloc (sizeof (gchar) * (size + 1));

  for (q = i = j = 0; i < size; i++)
    {
      if (*(tmp + i) == 0xD)
	continue;

      if (*(tmp + i) == 0xA || *(tmp + i) == 0x9 || *(tmp + i) == 0x20)
	{
	  if (!q)
	    {
	      q = 1;
	      ret[j++] = *(tmp + i);
	    }
	}

      else
	{
	  q = 0;
	  ret[j++] = *(tmp + i);
	}
    }

  ret[j] = 0;
  tmp = g_strdup (ret);
  g_free (ret);

  return tmp;
}

/* Libero memoria: */
void
rdf_free (struct rdf_t *rdf)
{
  if (!rdf)
    return;

  if (rdf->subject)
    g_free (rdf->subject);

  if (rdf->predicate)
    g_free (rdf->predicate);

  if (rdf->object)
    g_free (rdf->object);

  if (rdf->lang)
    g_free (rdf->lang);

  g_free (rdf);
}

/* Questa funzione crea un dump di una tripletta: */
struct rdf_t *
rdf_copy (struct rdf_t *rdf)
{
  struct rdf_t *new;

  if (!rdf)
    return NULL;

  new = (struct rdf_t *) g_malloc0 (sizeof (struct rdf_t));

  if (rdf->subject)
    new->subject = g_strdup (rdf->subject);

  if (rdf->predicate)
    new->predicate = g_strdup (rdf->predicate);

  if (rdf->object)
    new->object = g_strdup (rdf->object);

  if (rdf->lang)
    new->lang = g_strdup (rdf->lang);

  new->object_type = rdf->object_type;
  return new;
}

/* Questa funzione salva in un file una serie di triplette: */
gint
rdf_save_file (gchar * file, GList * rdf, GList ** namespace)
{
  FILE *fl;
  gchar *str;
  gchar *prefix, *prev = NULL, *tmp;
  GList *l;

  if (!(fl = g_fopen (file, "wb")))
    return 1;

  fprintf (fl, "<?xml version=\"1.0\"?>\n");

  prefix =
    markup (namespace_prefix
	    (namespace, "http://www.w3.org/1999/02/22-rdf-syntax-ns#"));

  fprintf (fl, "<%s:RDF\n", prefix);

  l = *namespace;
  while (l)
    {

      str = markup (((struct namespace_t *) l->data)->prefix);
      fprintf (fl, "   xmlns:%s=\"", str);
      g_free (str);

      str = markup (((struct namespace_t *) l->data)->namespace);
      fprintf (fl, "%s\"\n", str);
      g_free (str);

      l = l->next;
    }

  fprintf (fl, ">\n\n");

  if (rdf)
    {

      l = g_list_copy (rdf);
      l = g_list_sort (l, (GCompareFunc) rdf_compare);

      while (l)
	{
	  if (!prev || strcmp (prev, ((struct rdf_t *) l->data)->subject))
	    {

	      if (prev)
		fprintf (fl, "  </%s:Description>\n\n", prefix);

	      prev = ((struct rdf_t *) l->data)->subject;
	      str = markup (prev);

	      if (!rdf_blank (rdf, prev))
		fprintf (fl, "  <%s:Description %s:about=\"", prefix, prefix);
	      else
		fprintf (fl, "  <%s:Description %s:nodeID=\"", prefix,
			 prefix);

	      if (strcmp (str, THIS_DOCUMENT))
		fprintf (fl, "%s", str);

	      fprintf (fl, "\">\n");
	      g_free (str);
	    }

	  tmp =
	    namespace_create (*namespace,
			      ((struct rdf_t *) l->data)->predicate);

	  switch (((struct rdf_t *) l->data)->object_type)
	    {
	    case RDF_OBJECT_LITERAL:
	      str = markup (tmp);
	      fprintf (fl, "    <%s", str);
	      g_free (str);

	      if (((struct rdf_t *) l->data)->lang)
		{
		  str = markup (((struct rdf_t *) l->data)->lang);
		  fprintf (fl, " xml:lang=\"%s\"", str);
		  g_free (str);
		}

	      fprintf (fl, ">");

	      str = markup (((struct rdf_t *) l->data)->object);
	      fprintf (fl, "%s", str);
	      g_free (str);

	      str = markup (tmp);
	      fprintf (fl, "</%s>\n", str);
	      g_free (str);

	      break;
	    case RDF_OBJECT_RESOURCE:
	      str = markup (tmp);
	      fprintf (fl, "    <%s ", str);
	      g_free (str);

	      str = markup (((struct rdf_t *) l->data)->object);
	      fprintf (fl, "%s:resource=\"%s\" />\n", prefix, str);
	      g_free (str);
	      break;

	    case RDF_OBJECT_BLANK:
	      str = markup (tmp);
	      fprintf (fl, "    <%s ", str);
	      g_free (str);

	      str = markup (((struct rdf_t *) l->data)->object);
	      fprintf (fl, "%s:nodeID=\"%s\" />\n", prefix, str);
	      g_free (str);
	      break;
	    }

	  g_free (tmp);

	  l = l->next;
	}
    }

  if (prev)
    fprintf (fl, "  </%s:Description>\n\n", prefix);

  fprintf (fl, "</%s:RDF>\n\n", prefix);

  fclose (fl);

  g_free (prefix);
  return 0;
}

/* Questa funzione comunica se un nodo e' blank o no */
gboolean
rdf_blank (GList * rdf, gchar * t)
{
  struct rdf_t *r;

  while (rdf)
    {
      r = (struct rdf_t *) rdf->data;

      if (r->object_type == RDF_OBJECT_BLANK && !strcmp (r->object, t))
	return TRUE;
      rdf = rdf->next;
    }

  return FALSE;
}

/* Questa funzione comunica se un nodo e' una risorsa o no */
gboolean
rdf_resource (GList * rdf, gchar * t)
{
  struct rdf_t *r;

  while (rdf)
    {
      r = (struct rdf_t *) rdf->data;

      if (r->object_type == RDF_OBJECT_RESOURCE && !strcmp (r->object, t))
	return TRUE;
      rdf = rdf->next;
    }

  return FALSE;
}

struct data_t
{
  gchar *resource;
  GList *list;
  GList *nslist;

  gboolean this_document;
  gint flag;
  gint ret;
};

/* Questa funzione fa il lavoro sporco di parsing */
static void *
thread (void *what)
{
  GList *l, *ns;
  struct data_t *data = (struct data_t *) what;

  /* usa rdf_parse: */
  if (!(l = rdf_parse (data->resource, data->this_document)))
    {
      data->flag = 1;
      data->ret = 1;
      g_thread_exit (NULL);
      return NULL;
    }

  if (!(ns = namespace_parse (l)))
    {
      g_list_foreach (l, (GFunc) rdf_free, NULL);
      g_list_free (l);

      data->flag = 1;
      data->ret = 1;
      g_thread_exit (NULL);
      return NULL;
    }

  data->list = l;
  data->nslist = ns;
  data->flag = 1;
  data->ret = 0;

  g_thread_exit (NULL);
  return NULL;
}

/* Questa funzione viene lanciata quando il thread ha finito di compiere
 * le sue operazioni ma si e' premuto cancel. Libera la memoria: */
static void
rdf_thread_free (struct data_t *data)
{
  if (data->list)
    {
      g_list_foreach (data->list, (GFunc) rdf_free, NULL);
      g_list_free (data->list);
    }

  if (data->nslist)
    {
      g_list_foreach (data->nslist, (GFunc) namespace_free, NULL);
      g_list_free (data->nslist);
    }

  if (data->resource)
    g_free (data->resource);

  g_free (data);
}

/* Se si preme cancel si aggiunge un elemento alla lista dei thread da
 * gestire: */
static void
rdf_parse_thread_abort (GtkWidget * widget, struct data_t *data)
{
  struct thread_t *new;
  gboolean *ret;

  widget = gtk_widget_get_toplevel (widget);
  ret = g_object_get_data (G_OBJECT (widget), "ret");

  new = (struct thread_t *) g_malloc0 (sizeof (struct thread_t));

  new->func = (GFunc) rdf_thread_free;
  new->data = data;
  new->flag = &data->flag;

  thread_child = g_list_append (thread_child, new);
  *ret = 1;
}

/* Quando si chiede di parse qualcosa si usa questa perche' e' multi threading
 * e non fa bloccar e l'interfaccia: */
gboolean
rdf_parse_thread (gchar * resource, GList ** list, GList ** nslist,
		  gboolean this_document)
{
  GtkWidget *window;
  GtkWidget *pb;
  GThread *th;
  struct data_t *data;
  gboolean ret = 0;

  data = (struct data_t *) g_malloc (sizeof (struct data_t));

  data->resource = g_strdup (resource);
  data->this_document = this_document;
  data->list = NULL;
  data->nslist = NULL;
  data->flag = 0;

  window = dialog_wait (G_CALLBACK (rdf_parse_thread_abort), data, &pb);
  g_object_set_data (G_OBJECT (window), "ret", &ret);

  /* Attivo un processo separato ed apsetto: */
  th = g_thread_create ((GThreadFunc) thread, data, FALSE, NULL);

  while (!data->flag && !ret)
    {
      gtk_progress_bar_pulse (GTK_PROGRESS_BAR (pb));

      while (gtk_events_pending () && !data->flag && !ret)
	gtk_main_iteration ();

      g_usleep (50000);
    }

  gtk_widget_destroy (window);

  /* Questo succede se si e' cliccato sul pulsante abort: */
  if (ret)
    return 1;

  ret = data->ret;
  (*list) = data->list;
  (*nslist) = data->nslist;

  g_free (data->resource);
  g_free (data);

  return ret;
}

/* EOF */
